/**
 *	@file	IniFile.cpp
 *	@brief	init@C̓ǂݏ.
 *	@note
 *		IniFile.cpp ɍςm
 */

#include "stdafx.h"
#include "IniFile.h"
#include "DonutPFunc.h"



void CIniFileI::Close()
{
	m_strFileName.Empty();
	m_strSectionName.Empty();

  #if INISECTION_USE_MUTEX == 2
	delete		m_mutexLock;
	m_mutexLock = NULL;
  #endif
}



CIniFileI::CIniFileI(const CString &strFileName, const CString &strSectionName)
  #if INISECTION_USE_MUTEX == 2
	// : m_mutexName( DONUT_NAME _T("_IniFile_") + GetFileBaseName( strFileName ) )
	: m_mutexName( ConvPathNameToAlNumName(strFileName) )
	, m_mutex( NULL, false, m_mutexName )
	, m_mutexLock( new CMutexLock(m_mutex) )
  #endif
{
	Open(strFileName, strSectionName);
}



void CIniFileI::Open(const CString &strFileName, const CString &strSectionName)
{
	ATLASSERT( !strFileName.IsEmpty() );
	ATLASSERT( !IsOpen() );
	m_strFileName	 = strFileName;
	m_strSectionName = strSectionName;
}



///+++ F ValueName̎lԂ. G[̎AdwValue̒l͌̂܂.
LONG CIniFileI::QueryValue(DWORD &dwValue, LPCTSTR lpszValueName)
{
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );

	UINT nValue = ::GetPrivateProfileInt(m_strSectionName, lpszValueName, cnt_nDefault, m_strFileName);
	if (nValue == cnt_nDefault)
		return ERROR_CANTREAD;

	dwValue = nValue;
	return ERROR_SUCCESS;
}



#if 1
LONG CIniFileI::QueryValue(LPTSTR szValue, LPCTSTR lpszValueName, DWORD *pdwCount)
{
	ATLASSERT(pdwCount != NULL);
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );
	ATLASSERT(szValue != NULL); 		// Ini section can't determine dwCount.

	DWORD dw = ::GetPrivateProfileString(m_strSectionName, lpszValueName, _T(""), szValue, *pdwCount, m_strFileName);
	if (dw == 0)
		return ERROR_CANTREAD;

	return ERROR_SUCCESS;
}
#endif



#if 1	//+++ G[ԂӖȂꍇ̂ŁAG[pftHgw肠ŁAl̂̂Ԃo[Wp.
DWORD CIniFileI::GetValue(LPCTSTR lpszValueName, DWORD defalutValue)
{
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );
	return ::GetPrivateProfileInt(m_strSectionName, lpszValueName, defalutValue, m_strFileName);
}
#endif


#if 1	//+++ G[ԂӖȂꍇ̂ŁAG[pftHgw肠ŁAl̂̂Ԃo[Wp.

CString CIniFileI::GetString(LPCTSTR lpszValueName, const TCHAR* pszDefult, DWORD dwBufSize)
{
	enum { SIZE = MAX_PATH > 4096 ? MAX_PATH : 4096 };		//+++ 蔲ŌŒTCY.
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );

	if (dwBufSize != 0xFFFFFFFF) {
		ATLASSERT( dwBufSize <= SIZE );
		char buf[ SIZE ];
		if (dwBufSize == 0)
			dwBufSize  = SIZE;
		//x memset(buf, 0, sizeof buf);
		DWORD dw = ::GetPrivateProfileString(m_strSectionName, lpszValueName, _T(""), buf, dwBufSize, m_strFileName);
		if (dw == 0 && pszDefult)
			return CString(pszDefult);
		buf[dw] = '\0';
		return CString(buf);
	} else {
		DWORD	dwBufSize	= SIZE;
		DWORD	dw			= 0;
		TCHAR*	buf 		= "";
		for (;;) {
			buf = new TCHAR[ dwBufSize ];
			dw	= ::GetPrivateProfileString(m_strSectionName, lpszValueName, _T(""), buf, dwBufSize, m_strFileName);
			if (dw < dwBufSize - 1)
				break;
			delete[] buf;
			dwBufSize <<= 1;
		}
		if (dw == 0 && pszDefult) {
			delete[]	buf;
			return CString(pszDefult);
		}
		buf[dw] = '\0';
		CString 	str(buf);
		delete[]	buf;
		return str;
	}
}

#endif


// =================================================================
// ݊֌W

///+++ mutexŃK[hĂ݂
CIniFileIO::CIniFileIO(const CString &strFileName, const CString &strSectionName)
	: CIniFileI(strFileName, strSectionName)
  #if INISECTION_USE_MUTEX == 1
//	, m_mutexName( DONUT_NAME _T("_IniFile_") + GetFileBaseName( strFileName ) )
	, m_mutexName( ConvPathNameToAlNumName(strFileName) )
	, m_mutex( NULL, false, m_mutexName )
	, m_mutexLock( new CMutexLock(m_mutex) )
  #endif
{
}

///+++ 
void CIniFileIO::Close()
{
	CIniFileI::Close();
  #if INISECTION_USE_MUTEX == 1
	delete		m_mutexLock;
	m_mutexLock = NULL;
  #endif
}



///+++ obNAbvt@Cč폜.
void CIniFileIO::RemoveFileToBak()
{
	ATLASSERT( IsOpen() );
	MoveToBackupFile( m_strFileName );
}



LONG CIniFileIO::SetValue(DWORD dwValue, LPCTSTR lpszValueName)
{
	ATLASSERT(lpszValueName != NULL);
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );

	TCHAR szT[16];
	::wsprintf(szT, _T("%d"), dwValue);

	if ( ::WritePrivateProfileString(m_strSectionName, lpszValueName, szT, m_strFileName) )
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}



LONG CIniFileIO::SetString(LPCTSTR lpszValue, LPCTSTR lpszValueName)
{
	ATLASSERT(lpszValue != NULL);
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );

	ATLASSERT(m_strFileName.GetLength() < 4095);

	if ( ::WritePrivateProfileString(m_strSectionName, lpszValueName, lpszValue, m_strFileName) )
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}



LONG CIniFileIO::DeleteValue(LPCTSTR lpszValueName)
{
	ATLASSERT(lpszValueName != NULL);
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );
	if ( ::WritePrivateProfileString(m_strSectionName, lpszValueName, NULL, m_strFileName) )
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}



///+++ Init@C̃ZNV̍폜.
bool CIniFileIO::DeleteSection()
{
	ATLASSERT( IsOpen() );
	ATLASSERT( !m_strSectionName.IsEmpty() );

	return ::WritePrivateProfileString(m_strSectionName, NULL, NULL, m_strFileName) != 0;
}



#if 0
///+++ P .ini łȂ̃t@CƂĕ
void CIniFileIO::ForceWriteFile(const CString str)
{
	FILE* fp = fopen(m_strFileName, "wt");
	if (fp == NULL)
		return;
	//x size_t sz = str.GetLength();
	fputs( str, fp);
	fclose(fp);
}
#endif
